/*
Product Name: dhtmlxPortal 
Version: Beta 
Edition: Standard 
License: content of this file is covered by GPL. Usage outside GPL terms is prohibited. To obtain Commercial or Enterprise license contact sales@dhtmlx.com
Copyright UAB Dinamenta http://www.dhtmlx.com
*/

function dhtmlXPortal(conf) {
	
	var that = this;
	
	this.conf = {
		mode: (conf.mode=="w"?"w":"h"), // w/h
		skin: (conf.skin||window.dhx4.skin||(typeof(dhtmlx)!="undefined"?dhtmlx.skin:null)||window.dhx4.skinDetect("dhxportal")||"dhx_skyblue"),
		
		cell_size_params: {left: "marginLeft", top: "marginTop"}, // cell's _setSize params
		
		marker_w: 70, // width/height for drop-marker
		marker_zi: window.dhx4.newId(),
		marker_actv: null,
		
		dragable: true,
		live_drop: false,
		
		dnd_zi: window.dhx4.newId(),
		
		// fullscreen conf
		fs_mode: false,
		fs_tm: null,
		
		// base offsets
		ofs: {t:0,b:0,l:0,r:0},
		
		// some debug
		debug_drop_area: 0
	};
	
	// offset between cells
	this.conf.cell_ofs = window.dhx4.readFromCss("dhxportal_cell_ofs_"+this.conf.skin);
	
	this.rows = []; // rows
	this.cdata = {}; // cells
	this.markers = [];
	
	if (conf.parent == document.body) {
		var ofsDef = {
			dhx_skyblue: {t: 2, b: 2, l: 2, r: 2},
			dhx_web:     {t: 8, b: 8, l: 8, r: 8},
			dhx_terrace: {t: 9, b: 9, l: 8, r: 8}
		};
		this.conf.ofs = (ofsDef[this.conf.skin] != null ? ofsDef[this.conf.skin] : ofsDef.dhx_skyblue);
		this.conf.fs_mode = true;
	}
	
	// more offsets
	if (typeof(conf.offsets) != "undefined") {
		this.setOffsets(conf.offsets, false);
	}
	
	this.base = document.createElement("DIV");
	this.base.className = "dhxportal_cont";
	
	this.cont = (typeof(conf.parent)=="string"?document.getElementById(conf.parent):conf.parent);
	this.cont.className += " dhxportal_"+this.conf.skin;
	this.cont.appendChild(this.base);
	
	
	// items
	this._addRow = function(rowIndex) {
		
		var r = document.createElement("DIV");
		r.className = "dhxportal_single_row dhxportal_mode_"+this.conf.mode;
		
		if (this.base.childNodes[rowIndex] != null) {
			
			this.base.insertBefore(r, this.base.childNodes[rowIndex]);
			var r = this.rows;
			var q = 0;
			this.rows = [];
			
			while (q < r.length) {
				if (q == rowIndex) this.rows.push({row: r, cells: []});
				this.rows.push(r[q]);
				q++;
			}
			
		} else {
			
			this.base.appendChild(r);
			this.rows.push({row: r, cells: []});
			
		}
		r = null;
	}
	
	this._addCell = function(id, text, rowIndex, cellIndex) {
		
		this.cdata[id] = new dhtmlXPortalCell(id, this);
		this.cdata[id].conf.id = id;
		this.cdata[id].setText(text);
		this.cdata[id].cell._name = id; // for reorder after move
		
		var row = this.base.childNodes[rowIndex];
		if (row.childNodes[cellIndex] != null) {
			row.insertBefore(this.cdata[id].cell, row.childNodes[cellIndex]);
			this.rows[rowIndex].cells = [];
			for (var q=0; q<row.childNodes.length; q++) this.rows[rowIndex].cells.push(row.childNodes[q]._name);
		} else {
			row.appendChild(this.cdata[id].cell);
			this.rows[rowIndex].cells.push(id);
		}
		
		row = null;
	}
	
	this._moveCell = function(name, rowIndex, cellIndex, type) {
		
		if (type == "row") this._addRow(rowIndex);
		var row = this.base.childNodes[rowIndex];
		
		var oldRow = this.cdata[name].cell.parentNode;
		if (row.childNodes[cellIndex] != null) {
			row.insertBefore(this.cdata[name].cell, row.childNodes[cellIndex]);
		} else {
			row.appendChild(this.cdata[name].cell);
		}
		if (oldRow.childNodes.length == 0) oldRow.parentNode.removeChild(oldRow);
		row = oldRow = null;
		
		// rebuild rows
		while (this.rows.length > 0) {
			this.rows[0].row = this.rows[0].cells = null;
			this.rows[0] = null;
			this.rows.shift();
		}
		
		for (var q=0; q<this.base.childNodes.length; q++) {
			var cells = [];
			var row = this.base.childNodes[q];
			for (var w=0; w<row.childNodes.length; w++) cells.push(row.childNodes[w]._name);
			this.rows.push({ row: this.base.childNodes[q], cells: cells });
		}
		
		this._adjustCells();
	}
	
	// events
	window.dhx4._eventable(this);
	this._callMainEvent = function(name, args) {
		return this.callEvent(name, args);
	}
	
	// init self
	var rows = conf.rows;
	if (typeof(conf.pattern) == "string" && this.patterns[conf.pattern] != null) {
		rows = this.patterns[conf.pattern].rows;
		this.conf.mode = this.patterns[conf.pattern].mode;
	}
	if (rows != null) {
		for (var q=0; q<rows.length; q++) {
			
			this._addRow(q);
			
			for (var w=0; w<rows[q].length; w++) {
				
				var id = (typeof(rows[q][w].id)=="undefined"?window.dhx4.newId():String(rows[q][w].id));
				var text = (typeof(rows[q][w].text)=="undefined"?id:String(rows[q][w].text));
				this._addCell(id, text, q, w);
				
				if (typeof(rows[q][w].width) != "undefined") {
					this.cdata[id].conf.init_width = parseInt(rows[q][w].width);
				}
				if (typeof(rows[q][w].height) != "undefined") {
					this.cdata[id].conf.init_height = parseInt(rows[q][w].height);
				}
			}
		}
		rows = null;
		this.setSizes();
	}
	
	// dnd
	this._doOnMouseDown = function(e) {
		that._dndPrepare(e||event);
	}
	
	this._doOnMouseMove = function(e) {
		that._dndInAction(e||event);
	}
	
	this._doOnMouseUp = function(e) {
		that._dndStop(e);
	}
	
	this._doOnDragStart = function(e) {
		return that._dndStopDefDrag(e||event);
	}
	
	if (typeof(window.addEventListener) == "function") {
		this.base.addEventListener("mousedown", this._doOnMouseDown, false);
		this.base.addEventListener("dragstart", this._doOnDragStart, false);
	} else {
		this.base.attachEvent("onmousedown", this._doOnMouseDown);
		this.base.attachEvent("ondragstart", this._doOnDragStart);
	}
	
	// fullscreen resize
	if (this.conf.fs_mode == true) {
		
		this.conf.fs_tm = null;
		this._doOnResizeStart = function() {
			if (that.conf.fs_tm) window.clearTimeout(that.conf.fs_tm);
			that.conf.fs_tm = window.setTimeout(that._doOnResizeEnd, 200);
		}
		
		this._doOnResizeEnd = function() {
			that.setSizes();
		}
		
		if (typeof(window.addEventListener) == "function") {
			window.addEventListener("resize", this._doOnResizeStart, false);
		} else {
			window.attachEvent("onresize", this._doOnResizeStart);
		}
		
	}
	
	conf = null;
	
	this.unload = function() {
		
		this.conf.unloading = true;
		
		// dnd
		if (typeof(window.addEventListener) == "function") {
			this.base.removeEventListener("mousedown", this._doOnMouseDown, false);
			this.base.removeEventListener("dragstart", this._doOnDragStart, false);
		} else {
			this.base.detachEvent("onmousedown", this._doOnMouseDown);
			this.base.detachEvent("ondragstart", this._doOnDragStart);
		}
		
		// fullscreen resize
		if (this.conf.fs_mode == true) {
			if (typeof(window.addEventListener) == "function") {
				window.removeEventListener("resize", this._doOnResizeStart, false);
			} else {
				window.detachEvent("onresize", this._doOnResizeStart);
			}
		}
		
		// events
		window.dhx4._eventable(this, "clear");
		
		// cells
		for (var a in this.cdata) {
			this.cdata[a]._unload();
			this.cdata[a] = null;
		}
		
		// rows
		for (var q=0; q<this.rows.length; q++) {
			this.rows[q].row.parentNode.removeChild(this.rows[q].row);
			this.rows[q].row = this.rows[q].cells = null;
			this.rows[q] = null;
		}
		
		// clear base and cont
		this.cont.removeChild(this.base);
		var r = new RegExp("\s{0,}dhxportal_"+this.conf.skin, "gi");
		this.cont.className = this.cont.className.replace(r, "");
		this.cont = this.base = null;
		
		// conf
		for (var a in this.conf) this.conf[a] = null;
		
		// the rest
		for (var a in this) this[a] = null;
		that = null;
		
	}
	
	return this;
};

dhtmlXPortal.prototype.cells = function(name) {
	return this.cdata[name]||null;
};

dhtmlXPortal.prototype.forEachItem = function(func, env) {
	if (typeof(env) == "undefined") env = this;
	for (var a in this.cdata) {
		if (typeof(func) == "function") {
			func.apply(env, [this.cdata[a]]);
		} else if (typeof(func) == "string" && typeof(window[func]) == "function") {
			window[func].apply(env, [this.cdata[a]]);
		}
	}
	env = null;
};

dhtmlXPortal.prototype.addCell = function(rowIndex, cellIndex, id, createNewRow) {
	
	if (this.cdata[id] != null) return;
	
	if (rowIndex < 0 || cellIndex < 0) return;
	if (rowIndex > this.rows.length-1) rowIndex = this.rows.length-1;
	if (createNewRow == true) this._addRow(rowIndex);
	
	this._addCell(id, id, rowIndex, cellIndex);
	this._adjustCells();
};

// offsets
dhtmlXPortal.prototype.setOffsets = function(data, _upd) {
	var t = false;
	for (var a in data) {
		var k = a.charAt(0);
		if (typeof(this.conf.ofs[k]) != "undefined" && !isNaN(data[a])) {
			this.conf.ofs[k] = parseInt(data[a]);
			t = true;
		}
	}
	if (_upd !== false && typeof(this.setSizes) == "function" && t == true) this.setSizes();
};


// size
dhtmlXPortal.prototype.setSizes = function() {
	this._adjustBase();
	this._adjustCells();
};

dhtmlXPortal.prototype._adjustBase = function() {
	this.base.style.left = this.conf.ofs.l+"px";
	this.base.style.top = this.conf.ofs.t+"px";
	this.base.style.width = this.cont.offsetWidth-this.conf.ofs.l-this.conf.ofs.r+"px";
	this.base.style.height = this.cont.offsetHeight-this.conf.ofs.t-this.conf.ofs.b+"px";
};

dhtmlXPortal.prototype._adjustCells = function(base, pull, mode) {
	
	if (base == null) base = this.base;
	if (pull == null) pull = this.rows;
	if (mode == null) mode = this.conf.mode;
	
	if (pull == this.rows) {
		// check rows width/height
		var s = {};
		var a = (mode=="w"?"width":"height");
		for (var q=0; q<this.rows.length; q++) {
			s[q] = 0;
			for (var w=0; w<this.rows[q].cells.length; w++) {
				var id = this.rows[q].cells[w];
				s[q] = Math.max(s[q], this.cdata[id].conf["init_"+a]||0);
			}
		}
	}
	
	var totalRows = pull.length;
	var totalSize = 0;
	var baseSize = base[{w:"clientWidth", h:"clientHeight"}[mode]]
	var rowSize = Math.floor((baseSize-(totalRows-1)*this.conf.cell_ofs)/totalRows);
	
	for (var q=0; q<totalRows; q++) {
		
		var t = base.childNodes[q];
		var size = (q==totalRows-1?baseSize-totalSize-(q==0?0:this.conf.cell_ofs):rowSize);
		
		// margin for 2nd+ cell
		var marg = (q==0?0:this.conf.cell_ofs);
		
		if (t.className.indexOf("dhxportal_single_row") >= 0) {
			// row mode
			if (mode == "w") {
				t.style.width = size+"px";
				t.style.height = base.clientHeight+"px";
				t.style.marginTop = "0px";
				t.style.marginLeft = marg+"px";
			} else {
				t.style.height = size+"px";
				t.style.width = base.clientWidth+"px";
				t.style.marginLeft = "0px";
				t.style.marginTop = marg+"px";
			}
			if (window.dhx4.isIE8 == true) {
				t.style.filter = "alpha(opacity=99)";
				(function(t){window.setTimeout(function(){t.style.filter = "alpha(opacity=100)"; t = null;},1);})(t);
			}
		} else {
			// cell mode
			if (mode == "w") {
				this.cdata[pull[q]]._setSize(marg, 0, size, base.clientHeight, null, null, null, this.conf.cell_size_params);
			} else {
				this.cdata[pull[q]]._setSize(0, marg, base.clientWidth, size, null, null, null, this.conf.cell_size_params);
			}
		}
		
		totalSize += size+marg;
		
		// cells if any
		if (base == this.base && pull[q].cells.length > 0) {
			this._adjustCells(t, pull[q].cells, (this.conf.mode=="h"?"w":"h"));
		}
		
		t = null;
	}
};


window.dhtmlXPortalCell = function(id, portal) {
	
	dhtmlXCellObject.apply(this, [id, "_portal"]);
	
	var that = this;
	this.portal = portal;
	
	this.conf.skin = this.portal.conf.skin;
	
	this.attachEvent("_onCellUnload", function(){
		// dblclick header
		this.cell.childNodes[this.conf.idx.hdr].ondblclick = null;
		this.portal = null;
		that = null;
	});
	
	// init header
	this._hdrInit();
	this.cell.childNodes[this.conf.idx.hdr].ondblclick = function(){
		that.portal._callMainEvent("onDblClick", [that.conf.id]);
	};
	
	// onContentLoaded
	this.attachEvent("_onContentLoaded", function() {
		that.portal._callMainEvent("onContentLoaded", [this.conf.id]);
	});
	
	// for beta
	this._setSize = function(x, y, w, h, parentIdd, noCalcCont, actionType, customProps) {
		
		if (this.conf.size == null) this.conf.size = {};
		if (customProps == null) customProps = {}; // ability to use margin-left instead of left
		
		var styleProps = {left: "x", top: "y", width: "w", height: "h"};
		
		this.conf.size.x = x;
		this.conf.size.y = y;
		this.conf.size.w = Math.max(w,0);
		this.conf.size.h = Math.max(h,0);
		
		for (var a in styleProps) {
			var name = (customProps[a]||a);
			this.cell.style[name] = this.conf.size[styleProps[a]]+"px";
		}
		
		this.callEvent("_onSetSize",[]);
		
		if (noCalcCont !== true) {
			this._adjustCont(parentIdd, actionType);
		} else {
			this._adjustAttached(parentIdd);
		}
		
		this._adjustProgress();
	}
	
	return this;
	
};

dhtmlXPortalCell.prototype = new dhtmlXCellObject();

dhtmlXPortalCell.prototype.getId = function() {
	return this.conf.id;
};

// attach to cell
dhtmlXCellObject.prototype.attachPortal = function(conf) {
	
	var obj = document.createElement("DIV");
	obj.style.width = "100%";
	obj.style.height = "100%";
	obj.style.position = "relative";
	obj.style.overflow = "hidden";
	
	this._attachObject(obj);
	
	if (typeof(conf) == "string") {
		conf = {pattern: conf};
	} else if (typeof(conf) == "undefined") {
		conf = {};
	}
	conf.parent = obj;
	conf.skin = this.conf.skin;
	
	this.dataType = "portal";
	this.dataObj = new dhtmlXPortal(conf);
	
	conf.parent = obj = null;
	
	this.callEvent("_onContentAttach",[]);
	
	return this.dataObj;
	
};

dhtmlXPortalCell.prototype._hdrInit = function() {
	
	var cssExt = "";
	if (window.dhx4.isIE) {
		if (navigator.userAgent.indexOf("MSIE 8.0") != -1) {
			cssExt = " dhx_cell_hdr_text_ie8";
		} else if (navigator.userAgent.indexOf("MSIE 7.0") != -1) {
			cssExt = " dhx_cell_hdr_text_ie7";
		} else if (navigator.userAgent.indexOf("MSIE 6.0") != -1) {
			cssExt = " dhx_cell_hdr_text_ie6";
		}
	} else if (window.dhx4.isChrome) {
		cssExt = " dhx_cell_hdr_text_chrome";
	}
	
	var t = document.createElement("DIV");
	t.className = "dhx_cell_hdr";
	t.innerHTML = "<div class='dhx_cell_hdr_text"+cssExt+"'></div>";
	this.cell.insertBefore(t, this.cell.childNodes[this.conf.idx.cont]);
	t = null;
	
	// include into content top offset calculation
	this.conf.ofs_nodes.t._getHdrHeight = "func";
	
	// conf line
	this.conf.hdr = {visible: true};
	
	// include into index
	this.conf.idx_data.hdr = "dhx_cell_hdr";
	this._updateIdx();
	
	this.attachEvent("_onBorderChange", this._hdrUpdBorder);
	
};

dhtmlXPortalCell.prototype._getHdrHeight = function() {
	return this.cell.childNodes[this.conf.idx.hdr].offsetHeight;
};

// text
dhtmlXPortalCell.prototype.setText = function(text) {
	this.conf.hdr.text = text;
	this._hdrUpdText();
};

dhtmlXPortalCell.prototype.getText = function() {
	return this.conf.hdr.text;
};

dhtmlXPortalCell.prototype._hdrUpdText = function() {
	var text = this.conf.hdr.text;
	this.cell.childNodes[this.conf.idx.hdr].firstChild.innerHTML = "<span class='dhx_cell_hdr_text'>"+text+"</span>";
};

// show/hide
dhtmlXPortalCell.prototype.showHeader = function(noCalcCont) {
	
	if (this.conf.hdr.visible == true) return;
	
	this.conf.hdr.visible = true;
	this.cell.childNodes[this.conf.idx.hdr].className = "dhx_cell_hdr";
	
	if (noCalcCont !== true) this._adjustCont();
};

dhtmlXPortalCell.prototype.hideHeader = function(noCalcCont) {
	
	if (this.conf.hdr.visible == false) return;
	
	this.conf.hdr.visible = false;
	this.cell.childNodes[this.conf.idx.hdr].className = "dhx_cell_hdr dhx_cell_hdr_hidden";
	this._hdrUpdBorder();
	
	if (noCalcCont !== true) this._adjustCont();
};

dhtmlXPortalCell.prototype.isHeaderVisible = function() {
	return (this.conf.hdr.visible == true);
};

// border
dhtmlXPortalCell.prototype._hdrUpdBorder = function() {
	if (this.conf.borders == true) {
		this.cell.childNodes[this.conf.idx.hdr].className = "dhx_cell_hdr"+(this.conf.hdr.visible?"":" dhx_cell_hdr_hidden");
	} else {
		if (!this.conf.hdr.visible) {
			this.cell.childNodes[this.conf.idx.hdr].className = "dhx_cell_hdr dhx_cell_hdr_hidden_no_borders";
		}
	}
};


// dragger
dhtmlXPortal.prototype._initDndObj = function() {
	
	this._dndObj = document.createElement("DIV");
	this._dndObj.className = "dhx_cell_portal dhx_cell_dragged";
	this._dndObj.style.zIndex = window.dhx4.zim.reserve(this.conf.dnd_zi);
	this.base.appendChild(this._dndObj);
	
};

dhtmlXPortal.prototype._moveDndObj = function() {
	this._dndObj.style.left = this.conf.dnd.x-10+"px";
	this._dndObj.style.top = this.conf.dnd.y-9+"px";
};

dhtmlXPortal.prototype._clearDndObj = function() {
	if (this._dndObj != null) {
		window.dhx4.zim.clear(this.conf.dnd_zi);
		this._dndObj.parentNode.removeChild(this._dndObj);
		this._dndObj = null;
	}
};

// events
dhtmlXPortal.prototype._dndPrepare = function(e) {
	
	var t = e.target||e.srcElement;
	
	var cell = null;
	var allow = false;
	
	while (t != null) {
		if (t.className != null) {
			if (t.className.match(/^dhx_cell_portal/gi) != null) {
				if (allow == true) cell = t;
				t = null;
			} else if (t.className.match(/dhx_cell_hdr_text/gi) != null && t.parentNode != null && t.parentNode.parentNode != null && t.parentNode.parentNode.className.match(/^dhx_cell_portal/gi) != null) {
				// div.dhx_cell_portal -> div.dhx_cell_hdr -> div.dhx_cell_hdr_text -> span.dhx_cell_hdr_text
				cell = t.parentNode.parentNode;
				t = null;
			} else if (t.className.match(/dhx_cell_hdr/gi) != null && t.parentNode != null && t.parentNode.className.match(/^dhx_cell_portal/gi) != null) {
				// div.dhx_cell_portal -> div.dhx_cell_hdr -> div.dhx_cell_hdr_text
				cell = t.parentNode;
				t = null;
			} else if (t.className.match(/^dhxtabbar_tabs_cont/gi) != null || t.className.match(/^dhxtabbar_tab$/gi) != null || t.className.match(/^dhxtabbar_tab_text$/gi) != null) {
				// tabbar
				allow = true;
			}
		}
		if (t != null) t = t.parentNode;
	}
	
	if (cell != null) {
		
		if (e.preventDefault) e.preventDefault(); else e.returnValue = false; // cancel selection
		
		this.conf.dnd = {
			x: e.clientX-window.dhx4.absLeft(this.base),
			y: e.clientY-window.dhx4.absTop(this.base)
		};
		
		for (var q=0; q<this.rows.length; q++) {
			for (var w=0; w<this.rows[q].cells.length; w++) {
				var id = this.rows[q].cells[w];
				if (this.cdata[id].cell == cell) {
					this.conf.dnd.id = id;
					this.conf.dnd.cell = cell;
					this.conf.dnd.rowIndex = q;
					this.conf.dnd.cellIndex = w;
				}
			}
		}
		
		if (this._callMainEvent("onBeforeDrag", [this.conf.dnd.id]) !== true) {
			// deny by user
			this.conf.dnd.cell = cell = null;
			this.conf.dnd = null;
			return;
		}
		
		// dnd ready
		if (typeof(window.addEventListener) == "function") {
			window.addEventListener("mouseup", this._doOnMouseUp, false);
			window.addEventListener("mousemove", this._doOnMouseMove, false);
		} else {
			document.body.attachEvent("onmouseup", this._doOnMouseUp);
			document.body.attachEvent("onmousemove", this._doOnMouseMove);
			if (window.dhx4.isIE) {
				document.body.attachEvent("onselectstart", this._dndStopSelectIE);
			}
		}
		
	}
	
	dndObj = null;
};

dhtmlXPortal.prototype._dndInAction = function(e) {
	
	if (this.conf.dnd == null) return;
	
	if (this._dndObj == null) {
		
		this._dndFade = document.createElement("DIV");
		this._dndFade.className = "dhx_cell_fade";
		this.conf.dnd.cell.appendChild(this._dndFade);
		
		this._initMarkers();
		this._initDndObj();
		
	}
	
	this._moveDndObj();
	this._checkMarkers(e.clientX, e.clientY);
	
	this.conf.dnd.x = e.clientX-window.dhx4.absLeft(this.base);
	this.conf.dnd.y = e.clientY-window.dhx4.absTop(this.base);
};

dhtmlXPortal.prototype._dndStop = function() {
	
	if (typeof(window.addEventListener) == "function") {
		window.removeEventListener("mouseup", this._doOnMouseUp, false);
		window.removeEventListener("mousemove", this._doOnMouseMove, false);
	} else {
		document.body.detachEvent("onmouseup", this._doOnMouseUp);
		document.body.detachEvent("onmousemove", this._doOnMouseMove);
		if (window.dhx4.isIE) {
			document.body.detachEvent("onselectstart", this._dndStopSelectIE);
		}
	}
	
	if (this._dndFade != null) {
		this._dndFade.parentNode.removeChild(this._dndFade);
		this._dndFade = null;
	}
	
	// move cell
	var r = null;
	if (this.conf.live_drop != true && this.conf.marker_actv != null) {
		var m = this.markers[this.conf.marker_actv.index];
		r = {
			id: this.conf.dnd.id,
			cellIndex: m.cellIndex,
			rowIndex: m.rowIndex,
			type: m.type
		};
		this.conf.marker_actv = m = null;
	}
	
	this._clearDndObj();
	this._clearMarkers();
	
	for (var a in this.conf.dnd) this.conf.dnd[a] = null;
	this.conf.dnd = null;
	
	if (r != null) {
		this._moveCell(r.id, r.rowIndex, r.cellIndex, r.type);
		this._callMainEvent("onDrop", [r.id, r.rowIndex, r.cellIndex]);
	}
	
};

dhtmlXPortal.prototype._dndStopDefDrag = function(e) { // cancel default dnd
	
	var k = true;
	
	var t = e.target||e.srcElement;
	if (t.className != null && t.className.search(/dhx_cell_hdr_text/) >= 0) {
		e.cancelBubble = true;
		if (e.preventDefault) e.preventDefault(); else e.returnValue = false;
		k = false;
	}
	t = null;
	
	return k;
	
};

dhtmlXPortal.prototype._dndStopSelectIE = function(e) { // cancel selection in old IE
	e = e||event;
	e.returnValue = false;
};

dhtmlXPortal.prototype.enableLiveDrop = function(mode) {
	this.conf.live_drop = (typeof(mode)=="undefined"?true:window.dhx4.s2b(mode));
};

dhtmlXPortal.prototype._initMarkers = function() {
	
	this.conf.marker_actv = null;
	
	// for rows
	var isSingleInRow = (this.rows[this.conf.dnd.rowIndex].cells.length == 1);
	this._defineMarkers(this.rows, this.base, isSingleInRow);
	
	// ready-state marker
	if (this.rsMarker == null) {
		this.rsMarker = document.createElement("DIV");
		this.rsMarker.className = "dhxportal_marker_"+this.conf.skin;
		this.rsMarker.style.zIndex = window.dhx4.zim.reserve(this.conf.marker_zi);
		document.body.appendChild(this.rsMarker);
	}
	
};

// calculate marker pos and size
dhtmlXPortal.prototype._defineMarkers = function(pull, rowsParent, isSingle, cellRowIndex) {
	// check insert as a new row
	var t = {};
	
	var cellMode = (pull != this.rows);
	
	var prop = this.conf.dnd[(cellMode?"cellIndex":"rowIndex")];
	
	for (var q=0; q<pull.length; q++) {
		
		var before = !(q == prop && isSingle);
		var after = !(q == prop && isSingle);
		
		if (typeof(t[q]) == "undefined") t[q] = before; else t[q] = before && t[q];
		if (typeof(t[q+1]) == "undefined") t[q+1] = after; else t[q+1] = after && t[q+1];
		
	}
	
	for (var ax in t) {
		
		if (t[ax] == false) continue;
		
		var a = Number(ax);
		
		var mode = this.conf.mode;
		if (cellMode == true) mode = (mode=="w"?"h":"w");
		
		var w = this._calcMarkerPos(rowsParent, a, mode, (cellMode?"cell":"row"));
		
		w.x2 = w.left+w.width; // make some cache
		w.y2 = w.top+w.height;
		
		if (cellMode == true) {
			w.rowIndex = cellRowIndex;
			w.cellIndex = a;
		} else {
			w.rowIndex = a;
			w.cellIndex = 0;
		}
		
		this.markers.push(w);
		
		if (this.conf.debug_drop_area == true) {
			w.k = document.createElement("DIV");
			w.k.className = "debug_drop_area";
			w.k.style.left = w.left+"px";
			w.k.style.top = w.top+"px";
			w.k.style.width = w.width+"px";
			w.k.style.height = w.height+"px";
			document.body.appendChild(w.k);
		}

	}
	
	// cells
	if (cellMode != true) {
		for (var q=0; q<pull.length; q++) {
			this._defineMarkers(pull[q].cells, this.rows[q].row, (this.conf.dnd.rowIndex == q), q);
		}
	}
	
	pull = rowsParent = null;
};

dhtmlXPortal.prototype._calcMarkerPos = function(rowsParent, index, mode, type) {
	var w = {mode: mode, type: type};
	var k = rowsParent.childNodes[index];
	if (mode == "w") {
		var xOfs = Math.round((-this.conf.cell_ofs-this.conf.marker_w)/2);
		if (k == null) {
			k = rowsParent.lastChild;
			xOfs = k.offsetWidth-xOfs-this.conf.marker_w;
		}
		w.left = window.dhx4.absLeft(k)+xOfs;
		w.top = window.dhx4.absTop(k);
		w.width = this.conf.marker_w;
		w.height = k.offsetHeight;
	} else {
		var yOfs = Math.round((-this.conf.cell_ofs-this.conf.marker_w)/2);
		if (k == null) {
			k = rowsParent.lastChild;
			yOfs = k.offsetHeight-yOfs-this.conf.marker_w;
		}
		w.top = window.dhx4.absTop(k)+yOfs;
		w.left = window.dhx4.absLeft(k);
		w.width = k.offsetWidth;
		w.height = this.conf.marker_w;
	}
	k = null;
	return w;
};

dhtmlXPortal.prototype._clearMarkers = function() {
	while (this.markers.length > 0) {
		if (this.conf.debug_drop_area == true) {
			this.markers[0].k.parentNode.removeChild(this.markers[0].k);
			this.markers[0].k = null;
		}
		this.markers[0] = null;
		this.markers.shift();
	}
	// rs marker
	if (this.rsMarker != null) {
		window.dhx4.zim.clear(this.conf.marker_zi);
		this.rsMarker.parentNode.removeChild(this.rsMarker);
		this.rsMarker = null;
	}
};

// find marker for suitable drop zone while dnd
dhtmlXPortal.prototype._checkMarkers = function(x, y) {
	var index = -1;
	var current = 0;
	while (index < 0 && current < this.markers.length) {
		var m = this.markers[current];
		if (m.left <= x && x <= m.x2 && m.top <= y && y <= m.y2) {
			index = current;
		} else {
			current++;
		}
	}
	if (index >= 0) {
		if (this.conf.marker_actv == null) {
			this.conf.marker_actv = {index: -1, mode: null, css: false};
		}
		if (this.conf.marker_actv.index != index) {
			
			this.conf.marker_actv.index = index;
			if (this.callEvent("onDragIn", [this.conf.dnd.id, this.markers[index].rowIndex, this.markers[index].cellIndex]) !== true) return;
			
			if (this.conf.live_drop == true) {
				
				this.base.removeChild(this._dndObj);
				
				var r = {
					id: this.conf.dnd.id,
					rowIndex: this.markers[index].rowIndex,
					cellIndex: this.markers[index].cellIndex,
					type: this.markers[index].type
				};
				
				this._moveCell(r.id, r.rowIndex, r.cellIndex, r.type);
				
				this._clearMarkers();
				for (var q=0; q<this.rows.length; q++) {
					for (var w=0; w<this.rows[q].cells.length; w++) {
						var id = this.rows[q].cells[w];
						if (this.cdata[id].cell == this.conf.dnd.cell) {
							this.conf.dnd.rowIndex = q;
							this.conf.dnd.cellIndex = w;
						}
					}
				}
				
				this._initMarkers();
				this.base.appendChild(this._dndObj);
				
				this._callMainEvent("onDrop", [r.id, r.rowIndex, r.cellIndex]);
				
			} else {
				
				if (this.conf.marker_actv.css == false) {
					this.rsMarker.className = "dhxportal_marker_"+this.conf.skin+" dhxportal_marker_ready";
					this.conf.marker_actv.css = true;
				}
				if (this.conf.marker_actv.mode != this.markers[index].mode) {
					this.rsMarker.innerHTML = "<div class='dhxportal_marker_"+this.markers[index].mode+"'></div>";
					this.conf.marker_actv.mode = this.markers[index].mode;
				}
				for (var a in {top:1,left:1,width:1,height:1}) {
					this.rsMarker.style[a] = this.markers[index][a]+"px";
				}
				
			}
			
		}
		
	} else {
		if (this.conf.marker_actv != null) {
			for (var a in {top:1,left:1,width:1,height:1}) this.rsMarker.style[a] = "0px";
			if (this.conf.marker_actv.css == true) this.rsMarker.className = "dhxportal_marker_"+this.conf.skin;
			this.conf.marker_actv = null;
		}
	}
};

dhtmlXPortal.prototype.listPatterns = function() {
	var t = [];
	for (var a in this.patterns) t.push(a);
	return t;
};

dhtmlXPortal.prototype.patterns = {
	
	"1C": {mode: "w", rows: [[{id: "a"}]]},
	
	"2E": {mode: "h", rows: [[{id: "a"}], [{id: "b"}]]},
	"2U": {mode: "w", rows: [[{id: "a"}], [{id: "b"}]]},
	
	"3E": {mode: "h", rows: [[{id: "a"}], [{id: "b"}], [{id: "c"}]]},
	"3W": {mode: "w", rows: [[{id: "a"}], [{id: "b"}], [{id: "c"}]]},
	"3J": {mode: "w", rows: [[{id: "a"}, {id: "b"}], [{id: "c"}]]},
	"3L": {mode: "w", rows: [[{id: "a"}], [{id: "b"}, {id: "c"}]]},
	"3T": {mode: "h", rows: [[{id: "a"}], [{id: "b"}, {id: "c"}]]},
	"3U": {mode: "h", rows: [[{id: "a"}, {id: "b"}], [{id: "c"}]]},
	
	"4H": {mode: "w", rows: [[{id: "a"}], [{id: "b"}, {id: "c"}], [{id: "d"}]]},
	"4I": {mode: "h", rows: [[{id: "a"}], [{id: "b"}, {id: "c"}], [{id: "d"}]]},
	"4T": {mode: "h", rows: [[{id: "a"}], [{id: "b"}, {id: "c"}, {id: "d"}]]},
	"4U": {mode: "h", rows: [[{id: "a"}, {id: "b"}, {id: "c"}], [{id: "d"}]]},
	"4E": {mode: "h", rows: [[{id: "a"}], [{id: "b"}], [{id: "c"}], [{id: "d"}]]},
	"4W": {mode: "w", rows: [[{id: "a"}], [{id: "b"}], [{id: "c"}], [{id: "d"}]]},
	"4A": {mode: "w", rows: [[{id: "a"}, {id: "b"}], [{id: "c"}], [{id: "d"}]]},
	"4L": {mode: "w", rows: [[{id: "a"}], [{id: "b"}], [{id: "c"}, {id: "d"}]]},
	"4J": {mode: "h", rows: [[{id: "a"}], [{id: "b"}], [{id: "c"}, {id: "d"}]]},
	"4F": {mode: "h", rows: [[{id: "a"}, {id: "b"}], [{id: "c"}], [{id: "d"}]]},
	"4G": {mode: "w", rows: [[{id: "a"}, {id: "b"}, {id: "c"}], [{id: "d"}]]},
	"4C": {mode: "w", rows: [[{id: "a"}], [{id: "b"}, {id: "c"}, {id: "d"}]]},
	
	"5H": {mode: "w", rows: [[{id: "a"}], [{id: "b"}, {id: "c"}, {id: "d"}], [{id: "e"}]]},
	"5I": {mode: "h", rows: [[{id: "a"}], [{id: "b"}, {id: "c"}, {id: "d"}], [{id: "e"}]]},
	"5T": {mode: "h", rows: [[{id: "a"}], [{id: "b"}, {id: "c"}, {id: "d"}, {id: "e"}]]},
	"5U": {mode: "h", rows: [[{id: "a"}, {id: "b"}, {id: "c"}, {id: "d"}], [{id: "e"}]]},
	"5E": {mode: "h", rows: [[{id: "a"}], [{id: "b"}], [{id: "c"}], [{id: "d"}], [{id: "e"}]]},
	"5W": {mode: "w", rows: [[{id: "a"}], [{id: "b"}], [{id: "c"}], [{id: "d"}], [{id: "e"}]]},
	"5K": {mode: "w", rows: [[{id: "a"}, {id: "b"}, {id: "c"}], [{id: "d"}, {id: "e"}]]},
	"5S": {mode: "w", rows: [[{id: "a"}, {id: "b"}], [{id: "c"}, {id: "d"}, {id: "e"}]]},
	"5G": {mode: "w", rows: [[{id: "a"}, {id: "b"}, {id: "c"}], [{id: "d"}], [{id: "e"}]]},
	"5C": {mode: "w", rows: [[{id: "a"}], [{id: "b"}], [{id: "c"}, {id: "d"}, {id: "e"}]]},
	
	"6H": {mode: "w", rows: [[{id: "a"}], [{id: "b"}, {id: "c"}, {id: "d"}, {id: "e"}], [{id: "f"}]]},
	"6I": {mode: "h", rows: [[{id: "a"}], [{id: "b"}, {id: "c"}, {id: "d"}, {id: "e"}], [{id: "f"}]]},
	"6A": {mode: "w", rows: [[{id: "a"}, {id: "b"}, {id: "c"}, {id: "d"}, {id: "e"}], [{id: "f"}]]},
	"6C": {mode: "w", rows: [[{id: "a"}], [{id: "b"}, {id: "c"}, {id: "d"}, {id: "e"}, {id: "f"}]]},
	"6J": {mode: "w", rows: [[{id: "a"}, {id: "b"}, {id: "c"}, {id: "d"}], [{id: "e"}], [{id: "f"}]]},
	"6E": {mode: "w", rows: [[{id: "a"}], [{id: "b"}], [{id: "c"}, {id: "d"}, {id: "e"}, {id: "f"}]]},
	"6W": {mode: "w", rows: [[{id: "a"}], [{id: "b"}], [{id: "c"}], [{id: "d"}], [{id: "e"}], [{id: "f"}]]},
	
	"7H": {mode: "w", rows: [[{id: "a"}], [{id: "b"}, {id: "c"}, {id: "d"}, {id: "e"}, {id: "f"}], [{id: "g"}]]},
	"7I": {mode: "h", rows: [[{id: "a"}], [{id: "b"}, {id: "c"}, {id: "d"}, {id: "e"}, {id: "f"}], [{id: "g"}]]}
	
};

window.dhx4.readFromCss = function(className, property) {
	
	var t = document.createElement("DIV");
	t.className = className;
	if (document.body.firstChild != null) document.body.insertBefore(t, document.body.firstChild); else document.body.appendChild(t);
	
	var w = t[property||"offsetWidth"];
	t.parentNode.removeChild(t);
	t = null;
	
	return w;
};

window.dhx4.isIE8 = (navigator.userAgent.indexOf("MSIE 8.0") >= 0 && navigator.userAgent.indexOf("Trident") >= 0);

